const coral = @import("coral");

const ext = @import("./ext.zig");

const file = @import("./file.zig");

const kym = @import("./kym.zig");

pub const Manifest = struct {
	title: [255:0]coral.io.Byte = [_:0]coral.io.Byte{0} ** 255,
	width: u16 = 640,
	height: u16 = 480,
	tick_rate: f32 = 60.0,

	pub fn load(self: *Manifest, env: *kym.RuntimeEnv) kym.RuntimeError!void {
		const manifest = try env.import(file.Path.from(&.{"app.ona"})) orelse return;

		defer env.discard(manifest);

		const width = @as(u16, get: {
			if (try kym.get_field(env, manifest, "width")) |ref| {
				defer env.discard(ref);

				const fixed = try env.unbox_fixed(ref);

				if (fixed > 0 and fixed < coral.math.max_int(@typeInfo(@TypeOf(self.width)).Int)) {
					break: get @intCast(fixed);
				}
			}

			break: get self.width;
		});

		const height = @as(u16, get: {
			if (try kym.get_field(env, manifest, "height")) |ref| {
				defer env.discard(ref);

				const fixed = try env.unbox_fixed(ref);

				if (fixed > 0 and fixed < coral.math.max_int(@typeInfo(@TypeOf(self.height)).Int)) {
					break: get @intCast(fixed);
				}
			}

			break: get self.height;
		});

		const tick_rate = @as(f32, get: {
			if (try kym.get_field(env, manifest, "tick_rate")) |ref| {
				defer env.discard(ref);

				break: get @floatCast(try env.unbox_float(ref));
			}

			break: get self.tick_rate;
		});

		if (try kym.get_field(env, manifest, "title")) |ref| {
			defer env.discard(ref);

			const title_string = try env.unbox_string(ref);
			const limited_title_len = @min(title_string.len, self.title.len);

			coral.io.copy(&self.title, title_string[0 .. limited_title_len]);
			coral.io.zero(self.title[limited_title_len .. self.title.len]);
		} else {
			coral.io.zero(&self.title);
		}

		self.tick_rate = tick_rate;
		self.width = width;
		self.height = height;
	}
};

pub fn log_info(message: []const coral.io.Byte) void {
	ext.SDL_LogInfo(
		ext.SDL_LOG_CATEGORY_APPLICATION,
		"%.*s",
		coral.math.clamped_cast(@typeInfo(c_int).Int, message.len),
		message.ptr,
	);
}

pub fn log_warn(message: []const coral.io.Byte) void {
	ext.SDL_LogWarn(
		ext.SDL_LOG_CATEGORY_APPLICATION,
		"%.*s",
		coral.math.clamped_cast(@typeInfo(c_int).Int, message.len),
		message.ptr,
	);
}

pub fn log_fail(message: []const coral.io.Byte) void {
	ext.SDL_LogError(
		ext.SDL_LOG_CATEGORY_APPLICATION,
		"%.*s",
		coral.math.clamped_cast(@typeInfo(c_int).Int, message.len),
		message.ptr,
	);
}
